<?php

namespace Drupal\ckeditor_file_manager\Plugin\CKEditorPlugin;

use Drupal\ckeditor\CKEditorPluginBase;
use Drupal\ckeditor\CKEditorPluginConfigurableInterface;
use Drupal\ckeditor\CKEditorPluginContextualInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Entity\Editor;

/**
 * Defines the "CKEditor File Manager" plugin.
 *
 * @CKEditorPlugin(
 *   id = "jsplusFileManager",
 *   label = @Translation("CKEditor File Manager"),
 *   module = "ckeditor_file_manager"
 * )
 */
class ckeditor_file_manager extends CKEditorPluginBase implements CKEditorPluginConfigurableInterface, CKEditorPluginContextualInterface {

  public $plugins = array(
     'jsplusFileManager' => array(
    'buttons' => array(),
    'urlDoc' => 'http://js.plus/products/file-manager',
    'params' => array(
        array(
            'name' => 'skin',
            'inside' => 'jsplusFileManager',
            'default' => 'flat',
            'type' => 'str',
            'order' => 1000,
            'title' => 'Skin',
            'hint' => '',
            'widget' => 'select',
            'widgetOptions' => array( 'flat' => 'Flat', 'mono' => 'Mono', 'business' => 'Business'),
        ),
        array(
            'name' => 'skinMod',
            'inside' => 'jsplusFileManager',
            'default' => 'night',
            'type' => 'str',
            'order' => 1010,
            'title' => 'Skin modification',
            'hint' => '',
            'widget' => 'select',
            'widgetOptions' => array(
                                'night' => 'Night (for Flat only)', 'sea' => 'Sea (for Flat only)', 'snow' => 'Snow (for Flat only)',
                                'white' => 'White (for Mono only)', 'blue' => 'Blue (for Mono only)', 'violet' => 'Violet (for Mono only)',
                                '' => 'Default (for Business only)', 'icons2' => 'Icons2 (for Business only)'
                              ),
        ),
        array(
            'name' => 'showRootDir',
            'inside' => 'jsplusFileManager',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1020,
            'title' => 'Show root directory',
            'hint' => '',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'openLastDir',
            'inside' => 'jsplusFileManager',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1030,
            'title' => 'Open last directory',
            'hint' => '',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'dateFormat',
            'inside' => 'jsplusFileManager',
            'default' => 'dd/MM/yyyy HH:mm',
            'type' => 'str',
            'order' => 1040,
            'title' => 'Date format',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'hideCopyright',
            'inside' => 'jsplusFileManager',
            'default' => FALSE,
            'type' => 'bool',
            'order' => 1050,
            'title' => 'Hide branding',
            'hint' => '',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'returnUrlPrefix',
            'inside' => 'jsplusFileManager',
            'default' => '/libraries/jsplusFileManager/alphamanager/uploads/',
            'type' => 'str',
            'order' => 1060,
            'title' => 'Files URL prefix',
            'hint' => '',
            'widget' => 'text'
        )
    )
)
  );

    /**
     * {@inheritdoc}
     */
    public function getButtons() {
        if (!$this->isInstalled())
            return array();

        $buttons = array();
        foreach ($this->plugins as $pluginName => $pluginDef) {

            if (isset($pluginDef['buttons'])) {
                foreach($pluginDef['buttons'] as $buttonName => $buttonDef) {
                    $image = 'https://cdn.n1ed.com/cdn/buttons/' . $pluginName . '.png';
                    $button = array(
                        'label' => $buttonDef['label'],
                        'image' => $image
                    );
                    $buttons[$pluginName] = $button;
                }
            }

        }
        return $buttons;
    }

  /**
   * {@inheritdoc}
   */
  public function getFile() {
    return '/libraries/jsplusFileManager/plugin.js';
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function isInternal() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled(Editor $editor) {
    return TRUE;
  }

  function getConfigParam($settings, $param, $default, $type, $inside) {
    $name = $param;
    if (isset($settings[$name]) && strlen($settings[$name]) > 0)
        $value = $settings[$name];
    else
        $value = $default;
    if (isset($type) && $type == 'int') {
        $value = intval($value);
    } else if (isset($type) && $type == 'bool') {
        if ($value == '1')
            $value = true;
        else if ($value == '0')
            $value = false;
    } else if (isset($type) && $type == 'json') {
        $value = json_decode($value);
    }
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(Editor $editor) {
      $settings = array();
      if (isset($editor->getSettings()['plugins']['jsplusFileManager']))
        $settings = $editor->getSettings()['plugins']['jsplusFileManager'];
      $result = array();
      foreach ($this->plugins as $pluginName => $pluginDef) {
          if (isset($pluginDef['params'])) {
              foreach ($pluginDef['params'] as $paramDef) {
                  $value = $this->getConfigParam($settings, $paramDef['name'], $paramDef['default'], $paramDef['type'], isset($paramDef['inside']) ? $paramDef['inside'] : null);
                  if (!array_key_exists('inside', $paramDef))
                      $result[$paramDef['name']] = $value;
                  else {
                      if (!isset($result[$paramDef['inside']]))
                          $result[$paramDef['inside']] = array();
                      $result[$paramDef['inside']][$paramDef['name']] = $value;
                  }
              }
          }
      }
      return $result;
  }

  function addSelectToForm(& $form, $settings, $param, $title, $default, $options, $inside, $urlHelp) {
      $form[$param] = array(
        '#type' => 'select',
        '#title' => $title,
        '#options' => $options,
        '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
        '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
      );
  }

  function addTextboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textfield',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addTextareaToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textarea',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addCheckboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'checkbox',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'bool', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  public function isInstalled() {
    return file_exists($_SERVER['DOCUMENT_ROOT'] . '/libraries/jsplusFileManager');
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state, Editor $editor) {
    $editor_settings = $editor->getSettings();
    if (isset($editor_settings['plugins']['jsplusFileManager']))
      $settings = $editor_settings['plugins']['jsplusFileManager'];

    $form['#attached']['library'][] = 'ckeditor_file_manager/ckeditor_file_manager.admin';

    if (!$this->isInstalled()) {
        $form['warning'] = array(
            '#markup' => 'Looks like CKEditor File Manager (Drupal 8 module) is installed but CKEditor add-ons not found. In order to use this module please copy CKEditor add-ons into "libraries" forder in the root of your Drupal 8 installation (create the directory if it does not exist).'
        );
        return $form;
    }

    $params = array();
    foreach ($this->plugins as $pluginName => $pluginDef) {
        if (isset($pluginDef['params']))
            foreach ($pluginDef['params'] as $paramDef) {
                $paramDef['urlDoc'] = $pluginDef['urlDoc'];
                $params[$paramDef['order']] = $paramDef;
            }
    }
    ksort($params); // sort by key (order)

    if (count($params) > 0) {
        foreach ($params as $order => $paramDef) {
            $inside = isset($paramDef['inside']) ? $paramDef['inside'] : null;
            if ($paramDef['type'] == 'json')
                $paramDef['title'] = $paramDef['title'] . "<br/>\nImportant notice: this value needs to be in JSON format. So this is the must to use double quotes (not single ones) for all key names and all string values";
            if ($paramDef['widget'] == 'select') {
                $this->addSelectToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['widgetOptions'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'checkbox') {
                $this->addCheckboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'textarea') {
                $this->addTextareaToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else {
                $this->addTextboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            }
        }
    } else {
        //$settings = array();
        $form['warning'] = array(
            '#markup' => 'This add-on does not have parameters.'
        );
    }

    return $form;
  }

}
